#include <EEPROM.h>
#define EEPROM_SIZE 11 // 11 byte

#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27, 16, 2); // Set the LCD address to 0x27 for a 16 chars and 2 line display

#include <SoftwareSerial.h> 
#define DF_rxPin 2 // DFplayer RX -> arduino 2
#define DF_txPin 3 // DFplayer TX -> arduino 3 
SoftwareSerial dfSerial(DF_txPin, DF_rxPin); // (RX, TX)

#include <TimeLib.h>

// DFplayer command 변수
uint8_t df_command[] = {0x7E, 0xFF, 0x06, 0x00, 0x00, 0x00, 0x00, 0xEF};

void send_command(uint8_t cmd, uint8_t ack, uint8_t msb, uint8_t lsb) {
  df_command[3] = cmd;
  df_command[4] = ack;
  df_command[5] = msb;
  df_command[6] = lsb;
  for(int i = 0; i < sizeof(df_command); i++){
    dfSerial.write(df_command[i]);
  }
}

int h = 12; // initial Time display is 12:59:45 PM
int m = 56;
uint8_t s = 45;
bool meridian = true;
uint8_t hm;
bool now_am = true; //AM
uint16_t yy = 2019;
uint8_t mm = 1;
uint8_t dd = 1;
uint8_t week_num = 1;
String week_day = "SUN";
bool adjust = false;

bool display_t = false;
unsigned long int start_time = 0;
uint8_t volume = 20;

bool receive_status = false; // 질의 코드 0x42수신 여부 플래그
uint8_t status_count = 0;   // 질의값 저장 인덱스용 카운터
uint8_t status_val[7];         // 질의값 저장 배열
bool played = false;         // 질의결과 플래그 (재생중인지 아닌지)
bool status_check = false; // 질의 수신 완료 플래그

unsigned long int count_time = 0;
bool delay_set = false;
uint8_t sequence = 0;

bool speak = false; // 현재 상태 질의 시작 및 음성출력 시작 플래그
bool short_sentence = false;
bool sharp = false;

int am;
int time_sharp;
int hour_a;
int min_w;
int min_s;

uint8_t alarm_track_folder = 2;
uint8_t alarm_track = 2;

bool alarm_on = false;
int alarm_h;
int alarm_m;
unsigned long int alarm_delay = 0;
bool alarm_sentence = false;
uint8_t alarm_count = 0;
bool alarm_time = false;

unsigned long int alarm_sequence_delay = 0;
uint8_t alarm_sequence = 0;
bool alarm_send_f = false;
bool alarm_stop = false; 

bool no_speak_sharp = false;
uint8_t no_speak_s = 0;
uint8_t no_speak_f = 0;

uint8_t time_volume = 30; // 음성출력 및 알람 볼륨

uint8_t pin = 7;

void setup() {
  if(EEPROM.read(0) != 1) {
    EEPROM.write(0, 1); // eeprom 저장 플래그
    EEPROM.write(1, no_speak_sharp); // 묵음 플래그
    EEPROM.write(2, alarm_on); // 알람 플래그
    EEPROM.write(3, meridian); // 24 플래그
    EEPROM.write(4, no_speak_s); // 묵음 시작
    EEPROM.write(5, no_speak_f); // 묵음 종료
    EEPROM.write(6, alarm_h); // 알람 시
    EEPROM.write(7, alarm_m); // 알람 분
    EEPROM.write(8, alarm_track_folder); // 알람 폴더
    EEPROM.write(9, alarm_track); // 알람 트랙
    EEPROM.write(10, time_volume); // 알람 볼륨
  } else {
    no_speak_sharp = EEPROM.read(1);
    alarm_on = EEPROM.read(2);
    meridian = EEPROM.read(3);
    no_speak_s = EEPROM.read(4);
    no_speak_f = EEPROM.read(5);
    alarm_h = EEPROM.read(6);
    alarm_m = EEPROM.read(7);
    alarm_track_folder = EEPROM.read(8);
    alarm_track = EEPROM.read(9);
    time_volume = EEPROM.read(10);
  }
  Serial.begin(9600);
  dfSerial.begin(9600); // DFplayer Serial
  pinMode(pin, INPUT_PULLUP);  // pullup 저항 이용 - 버튼쪽에 저항 없음 OFF - HIHG, ON - LOW
  send_command(0x06, 0, 0, volume); // 볼륨 
  lcd.begin();
  lcd.backlight();
  lcd.clear();
  lcd.setCursor(0,0);  // row, col의 좌표로 커서를 위치 col: 0 ~ 15 row: 0, 1
  lcd.print("Arduino clock");
}

void loop() {
  cal_time();
  adjust_time();
  speak_time();
  delayCount();
  alarm_warning();
  alarm_send();
  if (dfSerial.available()) {   // 42 02 01
    uint8_t temp = dfSerial.read();
    if (temp == 0x42) receive_status = true; 
    if (receive_status == true) {
      status_val[status_count] = temp;
      status_count++; 
      if (temp == 0xEF) {
        status_check = true;
        receive_status = false;  
        status_count = 0; 
        if (status_val[2] == 2 && status_val[3] == 1) played = true; // 인덱스 2 = 2 -> SD카드, 인덱스 3 = 1 재생중
        else if (status_val[2] == 2 && status_val[3] == 0) played = false; // 인덱스 3 = 0 정지중
      }
    }
    else {
      Serial.write(temp);
    }
  }
  if (digitalRead(pin) == LOW) { // LOW - ON
    if (alarm_time == true) {
      alarm_stop = true;
      alarm_time = false;
      alarm_sentence = false;
      alarm_count = 0;
      alarm_send_f = true;
    }
    else {
      speak_option();
    }
  }
}

uint8_t vol_check = 0;
bool save_vol = false;
bool send_df = false; 

// 블루투스 앱 디지털 핀 제어용 변수
bool get_pin_val = false;
uint8_t pin_count = 0;
uint8_t pin_a[3] = {0, };
uint8_t pin_val;

// 블루투스 앱 상태표시 변수
uint8_t pin_echo[] = {0xF0, 0, 0xF1};

void adjust_time() {
  if (Serial.available() > 0) {
    if (Serial.peek() == 0x7E) send_df = true; 
    else if (Serial.peek() == 0xF0) get_pin_val = true; 
    if (send_df == true) {   // 안드로이드앱 데이터 수신 DFPlayer로 제어 코드 송신
      vol_check++;
      uint8_t temp = Serial.read(); 
      if (vol_check == 4 && temp == 0x06) save_vol = true;
      if (save_vol == true && vol_check == 7) { // 앱에서 변경된 볼륨값 저장
        volume = temp;
        save_vol = false;
      }
      if (temp == 0xEF) { send_df = false; vol_check = 0; }
      dfSerial.write(temp);  // send to DFplayer
    } 
    else if (get_pin_val == true) {  // 디지털 핀 제어값 수신
      uint8_t temp = Serial.read();
      pin_a[pin_count] = temp;
      pin_count++;
      if (temp == 0xF1) {
        get_pin_val = false;
        pin_count = 0;
        pin_control();
      }
    }
    else if (Serial.peek() == 0xF8) {  // 날짜 및 시간 값 수신
      String temp = Serial.readStringUntil('\n');
      String set = temp.substring(temp.indexOf("|")-10, temp.indexOf("|")-8);
      mm = set.toInt();
      set = temp.substring(temp.indexOf("|")-7, temp.indexOf("|")-5);
      dd = set.toInt();
      set = temp.substring(temp.indexOf("|")-4, temp.indexOf("|"));
      yy = set.toInt();
      set = temp.substring(temp.indexOf("|")+1, temp.indexOf("|")+3);
      h = set.toInt();
      set = temp.substring(temp.indexOf("|")+4, temp.indexOf("|")+6);
      m = set.toInt();
      set = temp.substring(temp.indexOf("|")+7, temp.indexOf("|")+9);
      s = set.toInt();
      setTime(h, m, s, dd, mm, yy); // time 라이브러리 이용, 현재 날짜를 유닉스 UTC로 변경
      week_num = weekday(); // time 라이브러리 이용, 현재 유닉스 UTC를 요일로 변환
      week_day_converter();
      start_time = millis();
      adjust = true;
      display_t = true;
      clock_status();
    }
    else if (Serial.peek() == 0xF9) {  // 알람 세팅 수신
      String temp = Serial.readStringUntil('\n');
      String set = temp.substring(1, temp.indexOf(":"));
      alarm_h = set.toInt();
      set = temp.substring(temp.indexOf(":") + 1, temp.length());
      alarm_m = set.toInt();
      EEPROM.write(6, alarm_h); // 알람 시
      EEPROM.write(7, alarm_m); // 알람 분
      Serial.print("alarm set "); Serial.print(alarm_h); Serial.print(":"); Serial.println(alarm_m);
    }
    else {
      String temp = Serial.readStringUntil('\n');
      if (temp == "dtime") display_time();
      else if (temp == "s") { s = 0; start_time = millis(); display_t = true; }
      else if (temp == "mm") { m++; display_t = true; }
      else if (temp == "m") { m--; display_t = true; }
      else if (temp == "hh") { h++; display_t = true; }
      else if (temp == "h") { h--; display_t = true; }
      else if (temp == "24") { meridian = !meridian; EEPROM.write(3, meridian); display_t = true; }
      else if (temp == "yy") { yy++; display_t = true; }
      else if (temp == "y") { yy--; display_t = true; }
      else if (temp == "mon") { mm++; adjust = true; display_t = true; }
      else if (temp == "mon-") { mm--; adjust = true; display_t = true; }
      else if (temp == "dd") { dd++; adjust = true; display_t = true; }
      else if (temp == "d") { dd--; adjust = true; display_t = true; }
      else if (temp == "ww") { week_num++; adjust = true; display_t = true; }
      else if (temp == "w") { week_num--; adjust = true; display_t = true; }
      else if (temp == "nsp") { no_speak_sharp = !no_speak_sharp; EEPROM.write(1, no_speak_sharp); }
      else if (temp.startsWith("atrack")) {
        String set = temp.substring(6, temp.indexOf(","));
        alarm_track_folder = set.toInt();
        set = temp.substring(temp.indexOf(",") + 1, temp.length());
        alarm_track = set.toInt();
        EEPROM.write(8, alarm_track_folder); // 알람 폴더
        EEPROM.write(9, alarm_track); // 알람 트랙
        Serial.print("Folder: "); Serial.print(alarm_track_folder); Serial.print(" ");
        Serial.print("Track: "); Serial.println(alarm_track); 
      }
      else if (temp.startsWith("avol")) {
        temp.remove(0, 4);
        time_volume = temp.toInt();
        EEPROM.write(10, time_volume); // 알람 볼륨
        Serial.print("alarm volume: "); Serial.println(time_volume); 
      }
      else if (temp.startsWith("tset")) {
        String set = temp.substring(4, 6);
        h = set.toInt();
        set = temp.substring(6, 8);
        m = set.toInt();
        set = temp.substring(8, 10);
        s = set.toInt();
        start_time = millis();
        adjust = true;
        display_t = true;
      }
      else if (temp.startsWith("dset")) {
        String set = temp.substring(4, 8);
        yy = set.toInt();
        set = temp.substring(8, 10);
        mm = set.toInt();
        set = temp.substring(10, 12);
        dd = set.toInt();
        adjust = true;
        display_t = true;
      }
      else if (temp == "alarm") { 
        alarm_on = !alarm_on;
        EEPROM.write(2, alarm_on); // 알람 플래그
        if (alarm_on) Serial.println("alarm on");
        else Serial.println("alarm off");
      }
      else if (temp.startsWith("aset")) {
        String set = temp.substring(4, 6);
        alarm_h = set.toInt();
        set = temp.substring(6, 8);
        alarm_m = set.toInt();
        EEPROM.write(6, alarm_h); // 알람 시
        EEPROM.write(7, alarm_m); // 알람 분
        Serial.print("alarm set "); Serial.print(alarm_h); Serial.print(":"); Serial.println(alarm_m); 
      }
      else if (temp.startsWith("setnsp")) {
        String set = temp.substring(6, temp.indexOf("~"));
        no_speak_s = set.toInt();
        set = temp.substring(temp.indexOf("~") + 1, temp.length());
        no_speak_f = set.toInt();
        EEPROM.write(4, no_speak_s); // 묵음 시작
        EEPROM.write(5, no_speak_f); // 묵음 종료
        Serial.print("no speak set: "); Serial.print(no_speak_s); Serial.print("-"); Serial.println(no_speak_f); 
      }
      else if (temp == "test") speak_option();
      else if (temp == "stop") { 
        alarm_stop = true;
        alarm_time = false;
        alarm_sentence = false;
        alarm_count = 0;
        alarm_send_f = true;
      }
      else if (temp == "play") send_command(0x0F, 0, 2, 6);
      else if (temp == "pause") send_command(0x0E, 1, 0, 0);
      else if (temp == "previous") send_command(0x02, 1, 0, 0);
      else if (temp == "next") send_command(0x01, 1, 0, 0);
      else if (temp == "vv") { if(volume < 30) volume++; send_command(0x06, 0, 0, volume); }
      else if (temp == "v") { if(volume > 0) volume--; send_command(0x06, 0, 0, volume); }
      else if (temp == "reset") send_command(0x0C, 0, 0, 0);
      if (m >= 60) m = 0;
      else if (m < 0) m = 59;
      if (h >= 24) h = 0; 
      else if (h < 0) h = 23; 
      if (h < 12) now_am = true;
      else now_am = false; 
      hm = h; 
      if(meridian == true && hm == 12) { now_am = false; }
      else if (meridian == true && hm > 12) { hm = hm - 12; }
      cal_dd();
      if (display_t == true) { display_time(); display_t = false; }
    }
  }
}

void clock_status() {
  if (no_speak_sharp == false) send_pin_echo(10);
  else send_pin_echo(11);
  if (alarm_on == false) send_pin_echo(20);
  else send_pin_echo(21);
  if (meridian == true) send_pin_echo(30);
  else send_pin_echo(31);
}

bool speak_finished = true;

void speak_option() {
  if (m == 0 && s != 0) {
    sharp = true;
    if (h < 13) time_sharp = 2200 + h;
    else time_sharp = 2200 + (h - 12);
  }
  else {
    sharp = false;
    if (h < 13) hour_a = 1000 + h;
    else hour_a = 1000 + (h - 12);
    if (h < 12) am = 1301;
    else am = 1302;
    int temp = m%10;
    if (temp == 0 || m < 10) {
      short_sentence = true;
      min_s = 1200 + m;
    }
    else {
      short_sentence = false;
      min_w = 1100 + (m - temp);
      min_s = 1200 + temp;
    }
  }
  speak = true;
  speak_finished = false;
}

uint8_t am_s = 4;
uint8_t vol_s = 6;
uint8_t hour_a_s = 21;
uint8_t long_min_w_s = 44;
uint8_t long_min_s_s = 65;
uint8_t long_min_s_end = 96;
uint8_t short_min_s_s = 44;
uint8_t short_min_s_end = 74;
uint8_t time_sharp_s = 4;
uint8_t time_sharp_end = 64;

void speak_time() {
  if (speak == true) {
    if (sequence == 0 && status_check == false) {
      send_command(0x42, 0, 0, 0);
      sequence++;
    }
    else if (sequence == 1 && status_check == true) {
      if (played == false) {
        send_command(0x0F, 0, 2, 255);
        sequence++; // 2
        delay_set = true;
      }
      else {
        delay_set = true;
        sequence = 3;
      }
    }
    else if (sequence >= am_s && sharp == false) {
      if (sequence == am_s) { advert(am); speak = false; }
      else if (sequence == vol_s) { send_command(0x06, 0, 0, time_volume); speak = false; }
      else if (sequence == hour_a_s) { advert(hour_a); speak = false; }
      if (short_sentence == false) {
        if (sequence == long_min_w_s) { advert(min_w); speak = false; }
        else if (sequence == long_min_s_s) { advert(min_s); speak = false; }
        else if (sequence == long_min_s_end) {
          send_command(0x06, 0, 0, volume);
          speak_finished = true;
          speak = false;
          status_check = false;
          sequence = 0;
        }
      }
      else {
        if (sequence == short_min_s_s) { advert(min_s); speak = false; }
        else if (sequence == vol_s) { send_command(0x06, 0, 0, time_volume); speak = false; }
        else if (sequence == short_min_s_end) {
          send_command(0x06, 0, 0, volume);
          speak_finished = true;
          speak = false;
          status_check = false;
          sequence = 0;
        }
      }
    }
    else if (sequence >= time_sharp_s && sharp == true) {
      if (sequence == time_sharp_s) { advert(time_sharp); speak = false; }
      else if (sequence == vol_s) { send_command(0x06, 0, 0, time_volume); speak = false; }
      else if (sequence == time_sharp_end) {
        send_command(0x06, 0, 0, volume);
        speak_finished = true;
        speak = false;
        status_check = false;
        sequence = 0;
        time_sharp_end = 64;
      }
    }
  }
}

void advert(int ad_num) {
  uint8_t ad_lsb = ad_num;
  uint8_t ad_msb = ad_num >> 8;
  send_command(0x13, 0, ad_msb, ad_lsb);
}

void delayCount() {
  if (delay_set == true) {
    if (millis() - count_time >= 50) { // 시간 간격: 밀리초
      count_time = millis();
      sequence++;
      if (sharp == false) { 
        if (sequence == am_s) speak = true;
        else if (sequence == vol_s) speak = true;
        else if (sequence == hour_a_s) speak = true;
        if (short_sentence == false) {
          if (sequence == long_min_w_s) speak = true;
          else if (sequence == long_min_s_s) speak = true;
          else if (sequence == long_min_s_end) { speak = true; delay_set = false; }
        }
        else {
          if (sequence == short_min_s_s) speak = true;
          else if (sequence == short_min_s_end) { speak = true; delay_set = false; }
        }
      }
      else {
        if (sequence == time_sharp_s) speak = true;
        else if (sequence == vol_s) speak = true;
        else if (sequence == time_sharp_end)  { speak = true; delay_set = false; }
      }
    }
  }
}

bool change_millis = false; 
uint16_t millis_sec = 994;

void cal_time() {
  if (millis() - start_time >= millis_sec) { // 시간 간격: 밀리초
    start_time = millis(); // 상기 조건을 만족할때의 밀리초를 다시 start_time에 저장하여 조건 초기화
    change_millis = ! change_millis;
    if (change_millis == true)  millis_sec = 995;
    else millis_sec = 994;
    s++;
    if (s == 60) {
      s = 0;
      m++;
    }
    if(m == 60) { m = 0; h++; }
    if(h == 12) { now_am = false;}
    else if (h == 24) { h = 0; now_am = true; dd++; week_num++; cal_dd(); }
    hm = h;
    if(meridian == true && hm >= 12) { now_am = false; }
    else now_am = true;
    if (meridian == true && hm > 12) { hm = hm - 12; }
    display_time();
    if (no_speak_sharp == true) {
      if (no_speak_s == no_speak_f) no_speak_sharp = false;
    }
    if (alarm_on) {
      if (alarm_h == h && alarm_m == m && s == 0){
        alarm_time = true;
        alarm_send_f = true;
      }
    }
    if (alarm_time != true) {
      if (m == 0 && s == 0) {
        if (no_speak_sharp == false) {
          speak_sharp();
        }
        else {
          if (no_speak_s > no_speak_f) {
            if (h < no_speak_s && h > no_speak_f) {
              speak_sharp();
            }
          }
          else if (no_speak_s < no_speak_f) {
            if (h > no_speak_s && h < no_speak_f) {
              speak_sharp();
            }
          }
        }
      }
    }
  }
}

void speak_sharp() {
  if (speak_finished == true) {
    sharp = true;
    if (h < 13) time_sharp = 2100 + h;
    else time_sharp = 2100 + (h - 12);
    time_sharp_end = 46;
    speak = true;   
  }
}

void alarm_send() {
  if (alarm_send_f == true) {
    if (millis() - alarm_sequence_delay >= 50) { // 시간 간격: 밀리초
      alarm_sequence_delay = millis();
      alarm_sequence++;
      if (alarm_stop == true) {
        if (alarm_sequence == 1) send_command(0x06, 0, 0, volume);
        else if (alarm_sequence == 2) send_command(0x16, 0, 0, 0); 
        else if (alarm_sequence == 3) {
          send_command(0x15, 0, 0, 0); 
          alarm_send_f = false;
          alarm_sequence = 0;
          alarm_stop = false;
        }
      }
      else {
        if (alarm_sequence == 1) send_command(0x16, 0, 0, 0);
        else if (alarm_sequence == 2) send_command(0x06, 0, 0, time_volume); // 볼륨 30
        else if (alarm_sequence == 3) send_command(0x0F, 0, alarm_track_folder, alarm_track); // 폴더2 002번 사운드 파일
        else if (alarm_sequence == 8) {
          send_command(0x19, 0, 0, 0); // 반복재생
          alarm_sentence = true;
          alarm_count = 0;
          alarm_send_f = false;
          alarm_sequence = 0;
        }
      }
    }
  }
}

void alarm_warning() {
  if (alarm_sentence == true) {
    if (millis() - alarm_delay >= 1000) { // 시간 간격: 밀리초
      alarm_delay = millis();
      alarm_count++;
      if (alarm_count == 4) {
        advert(1401); 
      }
      else if (alarm_count == 12) {
        advert(1402);
      }
      else if (alarm_count == 25) {
        advert(1403);
      }
      else if (alarm_count == 70) {
        advert(1404);
        alarm_sentence = false;
        alarm_count = 0;
      }
    }
  }
}

void cal_dd() {
  uint8_t m_dds;
  switch (mm) {
    case 1:   m_dds = 31; break;
    case 2:   m_dds = 28; break;
    case 3:   m_dds = 31; break;
    case 4:   m_dds = 30; break;
    case 5:   m_dds = 31; break;
    case 6:   m_dds = 30; break;
    case 7:   m_dds = 31; break;
    case 8:   m_dds = 31; break;
    case 9:   m_dds = 30; break;
    case 10:  m_dds = 31; break;
    case 11:  m_dds = 30; break;
    case 12:  m_dds = 31; break;
  }
  if (adjust == false) {
    if (dd > m_dds) { dd = 1; mm++; }
    if (mm > 12) { mm = 1; yy++; }
    if (week_num > 7) week_num = 1;
    week_day_converter();
  }
  else {
    if (dd > m_dds) dd = 1;
    else if (dd < 1) dd = m_dds;
    if (mm > 12) mm = 1;
    else if (mm < 1) mm = 12;
    if (week_num > 7) week_num = 1;
    else if (week_num < 1) week_num = 7; 
    week_day_converter();
    adjust = false;
  }
}

void week_day_converter() {
  switch (week_num) {
    case 1:   week_day = "SUN"; break;
    case 2:   week_day = "MON"; break;
    case 3:   week_day = "TUE"; break;
    case 4:   week_day = "WED"; break;
    case 5:   week_day = "THU"; break;
    case 6:   week_day = "FRI"; break;
    case 7:   week_day = "SAT"; break;
  }
}

void display_time() {
  lcd.setCursor(0,0); 
  lcd.print(yy); lcd.print("."); 
  if (mm < 10) { lcd.print("0"); lcd.print(mm); lcd.print("."); }
  else { lcd.print(mm); lcd.print("."); }
  if (dd < 10) { lcd.print("0"); lcd.print(dd); lcd.print(". "); }
  else { lcd.print(dd); lcd.print(". "); }
  lcd.print(week_day); lcd.print(" ");  
  lcd.setCursor(0,1);
  if (meridian == true) {
    if (now_am == true) lcd.print("AM ");
    else lcd.print("PM ");
    if (hm < 10) { lcd.print("0"); lcd.print(hm); lcd.print(":"); }
    else { lcd.print(hm); lcd.print(":"); }
  }
  else {
    if (h < 10) { lcd.print("0"); lcd.print(h); lcd.print(":"); }
    else { lcd.print(h); lcd.print(":"); }
  }
  if (m < 10) { lcd.print("0"); lcd.print(m); lcd.print(":"); }
  else { lcd.print(m); lcd.print(":"); }
  if (s < 10) { lcd.print("0"); lcd.print(s); lcd.print("     "); }
  else { lcd.print(s); lcd.print("     "); }  // 크리스탈 LCD에는 pritln 명령어 없음
}

void display_time_serial() {
  Serial.print(yy); Serial.print("."); 
  if (mm < 10) { Serial.print("0"); Serial.print(mm); Serial.print("."); }
  else { Serial.print(mm); Serial.print("."); }
  if (dd < 10) { Serial.print("0"); Serial.print(dd); Serial.print(". "); }
  else { Serial.print(dd); Serial.print(". "); }
  Serial.print(week_day); Serial.print(" ");  
  if (meridian == true) {
    if (now_am == true) Serial.print("AM ");
    else Serial.print("PM ");
    if (hm < 10) { Serial.print("0"); Serial.print(hm); Serial.print(":"); }
    else { Serial.print(hm); Serial.print(":"); }
  }
  else {
    if (h < 10) { Serial.print("0"); Serial.print(h); Serial.print(":"); }
    else { Serial.print(h); Serial.print(":"); }
  }
  if (m < 10) { Serial.print("0"); Serial.print(m); Serial.print(":"); }
  else { Serial.print(m); Serial.print(":"); }
  if (s < 10) { Serial.print("0"); Serial.println(s); }
  else Serial.println(s);
}

void pin_control() {
  pin_val = pin_a[1];
  if (pin_val == 11 && no_speak_s == no_speak_f) {
    pin_val = 0;
    send_pin_echo(10);
    Serial.print("Set NSP "); Serial.print(no_speak_s); Serial.print("-"); Serial.print(no_speak_f);
  }
  if (pin_val != 0) {
    switch (pin_val) {
      case 11: no_speak_sharp = true; 
               EEPROM.write(1, no_speak_sharp); // 묵음 플래그
               Serial.print("NSP ON ");
               Serial.print(no_speak_s);
               Serial.print("-");
               Serial.print(no_speak_f);
                  break;
      case 10: no_speak_sharp = false; 
               EEPROM.write(1, no_speak_sharp); // 묵음 플래그
               Serial.print("NSP OFF ");
               Serial.print(no_speak_s);
               Serial.print("-");
               Serial.print(no_speak_f);
                  break;
      case 21: alarm_on = true;
               EEPROM.write(2, alarm_on); // 알람 플래그
               Serial.print("Alarm ON ");
               Serial.print(alarm_h);
               Serial.print(":");
               Serial.print(alarm_m);
                  break;
      case 20: alarm_on = false;
               EEPROM.write(2, alarm_on); // 알람 플래그
               Serial.print("Alarm OFF ");
               Serial.print(alarm_h);
               Serial.print(":");
               Serial.print(alarm_m);
                  break;
      case 31: meridian = false;
               EEPROM.write(3, meridian); // 24 플래그
                  break;
      case 30: meridian = true;
               EEPROM.write(3, meridian); // 24 플래그
                  break;
      case 41: Serial.print("Folder: ");
               Serial.print(alarm_track_folder);
               Serial.print(" ");
               Serial.print("Track: ");
               Serial.print(alarm_track);
                  break;
      case 51: alarm_stop = true;
               alarm_time = false;
               alarm_sentence = false;
               alarm_count = 0;
               alarm_send_f = true;
                  break;
      case 61: Serial.print("alarm volume: ");
               Serial.print(time_volume);
                  break;
      case 71: speak_option();
               display_time_serial();
                  break;
    }
  pin_val = 0;
  }
}

void send_pin_echo(uint8_t pin_val){
  pin_echo[1] = pin_val;
  for (int i = 0; i < 3; i++) {
    Serial.write(pin_echo[i]);
  }
}
